using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;   // added for DllImport

namespace buffSrv
{
    class Property
    {
        public const short CF_INT4 = 515;
        public const short CF_FLOAT = 517;
        private const short CA_READ = 0x01;
        private const short CA_WRITE = 0x02;

        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void ServerCallback();

        [DllImport("TBufferedSrv.dll")]
        public static extern short RegisterServerCallback(string property, ServerCallback cb);
        [DllImport("TBufferedSrv.dll")]
        private static extern short pullBufferedData(string propertyName, string deviceName, int[] buffer, int propSize);
        [DllImport("TBufferedSrv.dll")]
        private static extern short pushBufferedData(string propertyName, string deviceName, int[] data, int propSize, int prpSchedule);
        [DllImport("TBufferedSrv.dll")]
        private static extern short RegisterBufferedProperty(string propertyName,
                                                            int prpInSize, short prpInFormat,
                                                            int prpOutSize, short prpOutFormat,
                                                            float prpMax, float prpMin,
                                                            string prpEgu,
                                                            short access,
                                                            string description);
        private string name;
        private string description;
        private string egu;
        private short accessMode;
        private int inSize;
        private int outSize;
        private short inFormat;
        private short outFormat;
        private float rangeMin;
        private float rangeMax;
        private ServerCallback callback;

        public Property(string name, string description, short outSize, short outFormat)
        {
            this.name = name;
            this.description = description;
            this.outFormat = outFormat;
            this.outSize = outSize;
            this.accessMode = CA_READ;
            callback = null;
        }
        public Property(string name, string description, short inSize, short inFormat, short outSize, short outFormat)
        {
            this.name = name;
            this.description = description;
            this.outFormat = outFormat;
            this.outSize = outSize;
            this.inFormat = inFormat;
            this.inSize = inSize;
            this.accessMode = CA_READ + CA_WRITE;
            callback = null;
        }

        public void setEgu(string egu) { this.egu = egu; }
        public void setRangeMin(float rangeMin) { this.rangeMin = rangeMin; }
        public void setRangeMax(float rangeMax) { this.rangeMax = rangeMax; }

        public short register()
        {
            return RegisterBufferedProperty(name, inSize, inFormat, outSize, outFormat,
                                                rangeMax, rangeMin, egu, accessMode,
                                                description);
        }
        public short attachCallback(ServerCallback callback)
        {
            this.callback = callback;
            return RegisterServerCallback(this.name, this.callback);  // use "this.callback" instead of "callback", to avoid GC problems
        }
        public short pushData(string deviceName, int[] data, int prpSchedule)
        {
            return pushBufferedData(this.name, deviceName, data, this.outSize, prpSchedule);
        }
        public short pullData(string deviceName, int[] data)
        {
            return pullBufferedData(this.name, deviceName, data, this.inSize);
        }
    }
}
